# FILE: mmDataTable.tcl
#
# Display Table of data values
#

# This code can be source'd from inside another script by setting
# no_net and no_event_loop as desired before sourcing this file.
# If no_net is set true, use the proc "AddTriples { triples }" to add
# data.

if {![info exists no_net]} {
    if {[string match "--nonet" [lindex $argv 0]]} {
	set no_net 1
    } else {
	set no_net 0      ;# Default is use net
    }
}

if {![info exists no_event_loop]} {
    set no_event_loop 0   ;# Default is to enter the event loop
}

# Support libraries
package require Oc
package require Tk
package require Ow
if {!$no_net} { package require Net }

Oc_Main SetAppName mmDataTable
Oc_Main SetVersion 1.0.1.0
regexp \\\044Date:(.*)\\\044 {$Date: 1998/10/05 20:37:06 $} _ date
Oc_Main SetDate [string trim $date]
# regexp \\\044Author:(.*)\\\044 {$Author: dgp $} _ author
# Oc_Main SetAuthor [Oc_Person Lookup [string trim $author]]
Oc_Main SetAuthor [Oc_Person Lookup donahue]
Oc_Main SetHelpURL [Oc_Url FromFilename [file join [file dirname \
        [file dirname [file dirname [Oc_DirectPathname [info \
        script]]]]] doc userguide Data_Table_Display_mmDataTa.html]]

# Death
proc Die { win } {
    if {![string match "." $win]} { return }  ;# Child destroy event
    global protocol
    if {[info exists protocol]} {
	$protocol Delete; after 2000 exit
    } else {
	exit
    }
}
bind . <Destroy> {+Die %W}

set menubar .mb
foreach {fmenu dmenu omenu hmenu} \
        [Ow_MakeMenubar . $menubar File Data Options Help] {}
$fmenu add command -label "Exit" -command { Die . } -underline 1
$dmenu configure -tearoff 1
$omenu add command -label "Format" -underline 0 \
	-command { ChangeFormatInteractive }
Ow_StdHelpMenu $hmenu

set menuwidth [Ow_GuessMenuWidth $menubar]
set brace [canvas .brace -width $menuwidth -height 0 -borderwidth 0 \
        -highlightthickness 0]
pack $brace -side top

set panel [frame .panel -relief groove]
pack $panel -side top -fill x -expand 1 -anchor nw
grid columnconfigure $panel 0 -weight 0
grid columnconfigure $panel 1 -weight 0
grid columnconfigure $panel 2 -weight 1

wm title . [Oc_Main GetAppName]<[pid]>
wm iconname . [wm title .]
Ow_SetIcon .

if {$tk_version>=8.0 && [string match windows $tcl_platform(platform)]} {
    # Windows doesn't size Tcl 8.0 menubar cleanly
    Oc_DisableAutoSize .
    wm geometry . "${menuwidth}x0"
    update
    wm geometry . {}
    Oc_EnableAutoSize .
    update
}

proc DisplayListAppend { label } {
    global panel unit onoff fmt value fmtvalue justify
    set foo [CompressLabel $label]
    if {$onoff($label)} {
	if {![winfo exists $panel.label$foo]} {
	    if {[string length $unit($label)]} {
		set txt "$label ($unit($label))"
	    } else {
		set txt "$label"
	    }
	    label $panel.label$foo -text $txt -anchor e
	}
	if {![winfo exists $panel.sep$foo]} {
	    label $panel.sep$foo -text ":"
	}
	if {![winfo exists $panel.value$foo]} {
	    catch {set fmtvalue($label) [format $fmt($label) $value($label)]}
	    label $panel.value$foo -textvariable fmtvalue($label)
	    if {![info exists justify($label)]} {
		set justify($label) $justify(default)
	    }
	    if {[string match "r" $justify($label)]} {
		$panel.value$foo configure -anchor e
	    } else {
		$panel.value$foo configure -anchor w
	    }
	    bind $panel.value$foo <Button-1> \
		    [list ChangeFormatInteractive $label]
	    bind $panel.value$foo <Button-3> \
		    [list ChangeJustifyInteractive $label]
	}
	grid $panel.label$foo $panel.sep$foo $panel.value$foo
	grid configure $panel.label$foo -sticky e
	grid configure $panel.value$foo -sticky ew
    } else {
	if {[winfo exists $panel.label$foo] && \
		[winfo exists $panel.value$foo]} {
	    grid forget $panel.label$foo $panel.value$foo $panel.sep$foo 
	    destroy $panel.label$foo $panel.value$foo $panel.sep$foo
	}
    }
    wm geometry . {}   ;# Set geometry to autosize
}

proc ChangeFormat { newformat {labellist {}} } {
    # Resets default format iff labellist is an empty list
    global fmt value fmtvalue onoff
    if {[llength $labellist]<1} {
	set lbllist [array names value]
	set fmt(default) $newformat
    } else {
	set lbllist $labellist
    }
    foreach lbl $lbllist {
	set fmt($lbl) $newformat
	if {[info exists onoff($lbl)] && $onoff($lbl) && \
	    [info exists value($lbl)] } {
	    catch {set fmtvalue($lbl) [format $newformat $value($lbl)]}
	}
    }
}

proc ChangeFormatInteractive { {label {}} } {
    global oommf fmt
    if {![string match {} $label] && [info exists fmt($label)]} {
	set format $fmt($label)
	set msg "Enter format string for label $label:"
	set labellist [list $label]
    } else {
	set format $fmt(default)
	set msg "Enter default numeric format string:"
	set labellist {}
    }
    set result [Ow_PromptDialog "[Oc_Main GetAppName]: Edit Format" \
	    $msg $format]
    if { [lindex $result 1] != 0 } { return }
    ChangeFormat [lindex $result 0] $labellist
}

proc ChangeJustify { newside {labellist {}} } {
    # Resets default justify iff labellist is an empty list
    global panel justify value fmtvalue onoff
    if {![string match "r" $newside]} { set newside "l" }
    if {[llength $labellist]<1} {
	set lbllist [array names value]
	set justify(default) $newside
    } else {
	set lbllist $labellist
    }
    foreach lbl $lbllist {
	set justify($lbl) $newside
	if {[info exists onoff($lbl)] && $onoff($lbl) && \
		[info exists value($lbl)] } {
	    set foo [CompressLabel $lbl]
	    if {[string match "r" $newside]} {
		$panel.value$foo configure -anchor e
	    } else {
		$panel.value$foo configure -anchor w
	    }
	}
    }
}

proc ChangeJustifyInteractive { {label {}} } {
    global oommf justify
    if {![string match {} $label] && [info exists justify($label)]} {
	set side $justify($label)
	set msg "Enter format justification (l|r) for label $label:"
	set labellist [list $label]
    } else {
	set side $justify(default)
	set msg "Enter default format justification (l|r):"
	set labellist {}
    }
    if {![string match "r" $side]} { set side "l" }
    set result [Ow_PromptDialog "[Oc_Main GetAppName]: Edit Justification" \
	    $msg $side]
    if { [lindex $result 1] != 0 } { return }
    set side [lindex $result 0]
    ChangeJustify $side $labellist
}


######################################################################
# PROC: AddTriples
#
# Data import routine for mmDataTable widget
# Parameters:
#  Each input consists of a triple of values: label, unit, value
# Data storage:
#  The first element of each import triple is used as an index into
# the following collection of global arrays:
#
#     unit($label)     :  Value units
#     fmt($label)      :  Format code to use for display
#     value($label)    :  Value
#     fmtvalue($label) :  Formatted value
#     onoff($label)    :  Whether or not value is currently displayed
#
#  The fmt global array also includes a "default" element which is the
# the format applied to new labels.
#
set fmt(default) "%g"    ;# Default format
set justify(default) "l" ;# Default justification
proc CompressLabel { lbl } {
    regsub -all "\[ \r\t\n\]" $lbl {} foo
    return $foo
}
proc AddTriples { triples } {
    global unit fmt value fmtvalue dmenu onoff
    foreach trip $triples {
        set lbl [lindex $trip 0]
	set value($lbl) [lindex $trip 2]
        if {![info exists unit($lbl)]} {
            set unit($lbl) [lindex $trip 1]
            set onoff($lbl) 0
	    set fmt($lbl) $fmt(default)
            regsub -all "\[\r\n\t\]" $lbl " " txt
            $dmenu add check -label $txt \
                    -variable onoff($lbl) \
                    -command [list DisplayListAppend $lbl]
        }
	if {$onoff($lbl)} {
	    catch {set fmtvalue($lbl) [format $fmt($lbl) $value($lbl)]}
	}
    }
}

if {!$no_net} {
    Net_Protocol New protocol -name "OOMMF DataTable protocol 0.0"
    $protocol AddMessage start HasGui {} {
	return [list start [list 0 0]]
    }
    $protocol AddMessage start DataTable { triples } {
	AddTriples $triples
	return [list start [list 0 0]]
    }
    Net_Server New server -protocol $protocol -alias [Oc_Main GetAppName] \
            -log Oc_Nop
    $server Start 0
}

if {!$no_event_loop} { vwait forever }
